@@ -1,12 +1,8 @@
-import logging
-import pdb
-
 import tensorflow as tf
 from tensorflow.keras.layers import Activation
 from typing import Union, Callable, Optional
 from tensorflow import Tensor
 from numpy import pi
-# import cvnn
 
 """
 This module contains many complex-valued activation functions to be used by CVNN class.
@@ -41,7 +37,7 @@ def modrelu(z: Tensor, b: float = 1., c: float = 1e-3) -> Tensor:
     return tf.cast(tf.keras.activations.relu(abs_z + b), dtype=z.dtype) * z / tf.cast(abs_z + c, dtype=z.dtype)
 
 
-def zrelu(z: Tensor) -> Tensor:
+def zrelu(z: Tensor, epsilon=1e-7) -> Tensor:
     """
     zReLU presented in "On Complex Valued Convolutional Neural Networks"
         from Nitzan Guberman (2016).
@@ -49,12 +45,12 @@ def zrelu(z: Tensor) -> Tensor:
 
     https://stackoverflow.com/questions/49412717/advanced-custom-activation-function-in-keras-tensorflow
     """
-    angle = tf.math.angle(z)
-    return tf.keras.backend.switch(0 <= angle,
-                                   tf.keras.backend.switch(angle <= pi / 2,
-                                                           z,
-                                                           tf.cast(0., dtype=z.dtype)),
-                                   tf.cast(0., dtype=z.dtype))
+    imag_relu = tf.nn.relu(tf.math.imag(z))
+    real_relu = tf.nn.relu(tf.math.real(z))
+    ret_real = imag_relu*real_relu / (imag_relu + epsilon)
+    ret_imag = imag_relu*real_relu / (real_relu + epsilon)
+    ret_val = tf.complex(ret_real, ret_imag)
+    return ret_val
 
 
 def crelu(z: Tensor, alpha: float = 0.0, max_value: Optional[float] = None, threshold: float = 0) -> Tensor:
@@ -80,6 +76,10 @@ Complex input, real output
 """
 
 
+def cast_to_real(z: Tensor) -> Tensor:
+    return tf.cast(z, z.dtype.real_dtype)
+
+
 def sigmoid_real(z: Tensor) -> Tensor:
     return tf.keras.activations.sigmoid(tf.math.real(z) + tf.math.imag(z))
 
@@ -534,6 +534,7 @@ def pol_selu(z: Tensor) -> Tensor:
 act_dispatcher = {
     'linear': Activation(linear),
     # Complex input, real output
+    'cast_to_real': Activation(cast_to_real),
     'convert_to_real_with_abs': Activation(convert_to_real_with_abs),
     'sigmoid_real': Activation(sigmoid_real),
     'softmax_real_with_abs': Activation(softmax_real_with_abs),
@@ -615,6 +616,6 @@ if __name__ == '__main__':
     
 
 __author__ = 'J. Agustin BARRACHINA'
-__version__ = '0.0.19'
+__version__ = '0.0.20'
 __maintainer__ = 'J. Agustin BARRACHINA'
 __email__ = 'joseagustin.barra@gmail.com; jose-agustin.barrachina@centralesupelec.fr'
