@@ -18,7 +18,7 @@ import cvnn.layers as layers
 import cvnn.dataset as dp
 from cvnn.data_analysis import MonteCarloAnalyzer, Plotter, get_confusion_matrix
 from cvnn.layers import ComplexDense, ComplexDropout
-from cvnn.utils import transform_to_real, randomize
+from cvnn.utils import transform_to_real, randomize, transform_to_real_map_function
 from cvnn.real_equiv_tools import get_real_equivalent
 from cvnn.utils import median_error
 # typing
@@ -50,7 +50,8 @@ class MonteCarlo:
             'summary_of_run': True,
             'tensorboard': False,
             'save_weights': False,
-            'safety_checkpoints': False
+            'safety_checkpoints': False,
+            'early_stop': False
         }
 
     def add_model(self, model: Type[Model]):
@@ -62,7 +63,8 @@ class MonteCarlo:
     def run(self, x, y, data_summary: str = '',
             real_cast_modes: Optional[Union[str, List[Optional[str]], Tuple[Optional[str]]]] = None,
             validation_split: float = 0.2,
-            validation_data: Optional[Union[Tuple[np.ndarray, np.ndarray], data.Dataset]] = None,  # TODO: Add the tuple of validation data details.
+            validation_data: Optional[Union[Tuple[np.ndarray, np.ndarray], data.Dataset]] = None,
+            # TODO: Add the tuple of validation data details.
             test_data: Optional[Union[Tuple[np.ndarray, np.ndarray], data.Dataset]] = None,
             iterations: int = 100, epochs: int = 10, batch_size: int = 100,
             shuffle: bool = True, debug: bool = False, display_freq: int = 1):
@@ -123,16 +125,13 @@ class MonteCarlo:
         real_cast_modes = self._check_real_cast_modes(real_cast_modes)
         confusion_matrix, pbar, test_results = self._beginning_callback(iterations, epochs, batch_size,
                                                                         shuffle, data_summary, test_data_cols)
-        x, y = randomize(x, y)
-        w_save = []                     # TODO: Find a better method
-        for model in self.models:       # ATTENTION: This will make all models have the SAME weights, not ideal
-            w_save.append(model.get_weights())     # Save model weight
+        w_save = []  # TODO: Find a better method
+        for model in self.models:  # ATTENTION: This will make all models have the SAME weights, not ideal
+            w_save.append(model.get_weights())  # Save model weight
         # np.save(self.monte_carlo_analyzer.path / "initial_debug_weights.npy", np.array(w_save))     # TODO
         for it in range(iterations):
             if debug:
                 logger.info("Iteration {}/{}".format(it + 1, iterations))
-            if shuffle:  # shuffle all data at each iteration
-                x, y = randomize(x, y)
             for i, model in enumerate(self.models):
                 x_fit, val_data_fit, test_data_fit = self._get_fit_dataset(model.inputs[0].dtype.is_complex, x,
                                                                            validation_data, test_data,
@@ -140,15 +139,18 @@ class MonteCarlo:
                 model.set_weights(w_save[i])
                 temp_path = self.monte_carlo_analyzer.path / f"run/iteration{it}_model{i}_{model.name}"
                 os.makedirs(temp_path, exist_ok=True)
-                callbacks = None
+                callbacks = []
                 if self.output_config['tensorboard']:
                     tensorboard_callback = tf.keras.callbacks.TensorBoard(log_dir=temp_path / 'tensorboard',
                                                                           histogram_freq=1)
-                    callbacks = [tensorboard_callback]
+                    callbacks.append(tensorboard_callback)
+                if self.output_config['early_stop']:
+                    eas = tf.keras.callbacks.EarlyStopping(monitor='val_loss', patience=10)
+                    callbacks.append(eas)
                 run_result = model.fit(x_fit, y, validation_split=validation_split, validation_data=val_data_fit,
                                        epochs=epochs, batch_size=batch_size,
                                        verbose=debug, validation_freq=display_freq,
-                                       callbacks=callbacks)
+                                       callbacks=callbacks, shuffle=shuffle)
                 test_results = self._inner_callback(model, validation_data, confusion_matrix, real_cast_modes[i], i,
                                                     run_result, test_results, test_data_fit, temp_path)
             self._outer_callback(pbar)
@@ -166,20 +168,39 @@ class MonteCarlo:
         return real_cast_modes
 
     @staticmethod
-    def _get_fit_dataset(is_complex: bool, x, validation_data, test_data, polar):
-        # TODO: This does not work with tf.Dataset.
+    def _transform_dataset(is_complex: bool, validation_data, polar):
         val_data_fit = None
-        test_data_fit = None
-        if is_complex:
+        if validation_data is not None:
+            if isinstance(validation_data, tf.data.Dataset):
+                if not is_complex:
+                    val_data_fit = validation_data.map(lambda x, y: transform_to_real_map_function(x, y, mode=polar))
+                else:
+                    val_data_fit = validation_data
+            elif (is_complex and validation_data[0].dtype.is_complex) or \
+                    (not is_complex and validation_data[0].dtype.is_floating):
+                val_data_fit = validation_data
+            elif is_complex and not validation_data[0].dtype.is_complex:
+                raise NotImplementedError(f"The input dataset is expected to be complex")
+            else:
+                val_data_fit = (transform_to_real(validation_data[0], mode=polar), validation_data[1])
+        return val_data_fit
+
+    def _get_fit_dataset(self, is_complex: bool, x, validation_data, test_data, polar):
+        if isinstance(x, tf.data.Dataset):
+            if not is_complex:
+                x_fit = x.map(lambda imag, label: transform_to_real_map_function(imag, label, mode=polar))
+            else:
+                x_fit = x
+        elif (is_complex and tf.dtypes.as_dtype(x.dtype).is_complex) or \
+                (not is_complex and tf.dtypes.as_dtype(x.dtype).is_floating):
             x_fit = x
-            val_data_fit = validation_data
-            test_data_fit = test_data
+        elif is_complex and not tf.dtypes.as_dtype(x.dtype).is_complex:
+            raise NotImplementedError(f"Cast real dataset to complex not yet implemented, "
+                                      f"please provide the dataset in complex form.")
         else:
             x_fit = transform_to_real(x, mode=polar)
-            if validation_data is not None:
-                val_data_fit = (transform_to_real(validation_data[0], mode=polar), validation_data[1])
-            if test_data is not None:
-                test_data_fit = (transform_to_real(test_data[0], mode=polar), test_data[1])
+        val_data_fit = self._transform_dataset(is_complex, validation_data, polar)
+        test_data_fit = self._transform_dataset(is_complex, test_data, polar)
         return x_fit, val_data_fit, test_data_fit
 
     # Callbacks
@@ -233,7 +254,8 @@ class MonteCarlo:
                     strs.sort()
                     cm_sorted = cm.fillna(0)[ints + strs]  # Sorted confusion matrix
                     model_cm['matrix'] = cm_sorted.groupby(cm_sorted.index).mean()
-                    model_cm['matrix'].to_csv(self.monte_carlo_analyzer.path / (model_cm['name'] + "_confusion_matrix.csv"))
+                    model_cm['matrix'].to_csv(
+                        self.monte_carlo_analyzer.path / (model_cm['name'] + "_confusion_matrix.csv"))
         if test_results is not None:
             test_results.groupby('network').describe().to_csv(self.monte_carlo_analyzer.path / ("test_results.csv"))
         if self.output_config['plot_all']:
@@ -277,13 +299,13 @@ class MonteCarlo:
             self.pandas_full_data.to_csv(self.monte_carlo_analyzer.path / "run_data.csv", index=False)
 
     # Saver functions
-    def _save_montecarlo_log(self, iterations, dataset_name,  num_classes, polar_mode, dataset_size,
+    def _save_montecarlo_log(self, iterations, dataset_name, num_classes, polar_mode, dataset_size,
                              features_size, epochs, batch_size):
         fieldnames = [
             'iterations',
-            'dataset', '# Classes', "Dataset Size", 'Feature Size',     # Dataset information
-            'models', 'epochs', 'batch size', "Polar Mode",             # Models information
-            'path', "cvnn version"                                      # Library information
+            'dataset', '# Classes', "Dataset Size", 'Feature Size',  # Dataset information
+            'models', 'epochs', 'batch size', "Polar Mode",  # Models information
+            'path', "cvnn version"  # Library information
         ]
         row_data = [
             iterations,
@@ -406,13 +428,13 @@ class RealVsComplex(MonteCarlo):
 
 def run_gaussian_dataset_montecarlo(iterations: int = 30, m: int = 10000, n: int = 128, param_list=None,
                                     epochs: int = 300, batch_size: int = 100, display_freq: int = 1,
-                                    optimizer='sgd', validation_split: float = 0.2,      # TODO: Add typing here
+                                    optimizer='sgd', validation_split: float = 0.2,  # TODO: Add typing here
                                     shape_raw: List[int] = None, activation: t_activation = 'cart_relu',
                                     debug: bool = False, do_all: bool = True, tensorboard: bool = False,
                                     polar: Optional[Union[str, List[Optional[str]], Tuple[Optional[str]]]] = None,
                                     capacity_equivalent: bool = True, equiv_technique: str = 'ratio',
                                     dropout: Optional[float] = None, models: Optional[List[Model]] = None,
-                                    plot_data: bool = True) -> str:
+                                    plot_data: bool = True, early_stop: bool = False) -> str:
     """
     This function is used to compare CVNN vs RVNN performance over statistical non-circular data.
         1. Generates a complex-valued gaussian correlated noise with the characteristics given by the inputs.
@@ -470,7 +492,7 @@ def run_gaussian_dataset_montecarlo(iterations: int = 30, m: int = 10000, n: int
                               iterations=iterations, epochs=epochs, batch_size=batch_size, display_freq=display_freq,
                               validation_split=validation_split, validation_data=None,
                               debug=debug, polar=polar, do_all=do_all, tensorboard=tensorboard, do_conf_mat=False,
-                              plot_data=plot_data)
+                              plot_data=plot_data, early_stop=early_stop)
     else:
         return mlp_run_real_comparison_montecarlo(dataset=dataset, open_dataset=None, iterations=iterations,
                                                   epochs=epochs, batch_size=batch_size, display_freq=display_freq,
@@ -487,10 +509,12 @@ def run_montecarlo(models: List[Model], dataset: cvnn.dataset.Dataset, open_data
                    iterations: int = 30,
                    epochs: int = 300, batch_size: int = 100, display_freq: int = 1,
                    validation_split: float = 0.2,
-                   validation_data: Optional[Union[Tuple, data.Dataset]] = None,  # TODO: Add vallidation data tuple details
-                   debug: Union[bool, int] = False, do_conf_mat: bool = False, do_all: bool = True, tensorboard: bool = False,
+                   validation_data: Optional[Union[Tuple, data.Dataset]] = None,
+                   # TODO: Add vallidation data tuple details
+                   debug: Union[bool, int] = False, do_conf_mat: bool = False, do_all: bool = True,
+                   tensorboard: bool = False,
                    polar: Optional[Union[str, List[Optional[str]], Tuple[Optional[str]]]] = None,
-                   plot_data: bool = False) -> str:
+                   plot_data: bool = False, early_stop: bool = True) -> str:
     """
     This function is used to compare different neural networks performance.
     1. Runs simulation and compares them.
@@ -538,6 +562,7 @@ def run_montecarlo(models: List[Model], dataset: cvnn.dataset.Dataset, open_data
     monte_carlo.output_config['tensorboard'] = tensorboard
     monte_carlo.output_config['confusion_matrix'] = do_conf_mat
     monte_carlo.output_config['plot_all'] = do_all
+    monte_carlo.output_config['early_stop'] = early_stop
     if plot_data:
         dataset.plot_data(overlapped=True, showfig=False, save_path=monte_carlo.monte_carlo_analyzer.path,
                           library='matplotlib')
@@ -563,13 +588,14 @@ def run_montecarlo(models: List[Model], dataset: cvnn.dataset.Dataset, open_data
 def mlp_run_real_comparison_montecarlo(dataset: cvnn.dataset.Dataset, open_dataset: Optional[t_path] = None,
                                        iterations: int = 30,
                                        epochs: int = 300, batch_size: int = 100, display_freq: int = 1,
-                                       optimizer='adam',     # TODO: Typing
+                                       optimizer='adam',  # TODO: Typing
                                        shape_raw=None, activation: t_activation = 'cart_relu',
-                                       output_activation : t_activation = DEFAULT_OUTPUT_ACT,
-                                       debug:  Union[bool, int] = False, do_all: bool = True,
+                                       output_activation: t_activation = DEFAULT_OUTPUT_ACT,
+                                       debug: Union[bool, int] = False, do_all: bool = True,
                                        polar: Optional[Union[str, List[Optional[str]], Tuple[Optional[str]]]] = None,
                                        dropout: float = 0.5, validation_split: float = 0.2,
-                                       validation_data: Optional[Union[Tuple, data.Dataset]] = None,    # TODO: Add typing of tuple
+                                       validation_data: Optional[Union[Tuple, data.Dataset]] = None,
+                                       # TODO: Add typing of tuple
                                        capacity_equivalent: bool = True, equiv_technique: str = 'ratio',
                                        shuffle: bool = False, tensorboard: bool = False, do_conf_mat: bool = False,
                                        plot_data: bool = True) -> str:
@@ -778,9 +804,9 @@ def _save_montecarlo_log(iterations, path, dataset_name, models_names, num_class
                          features_size, epochs, batch_size, filename=None):
     fieldnames = [
         'iterations',
-        'dataset', '# Classes', "Dataset Size", 'Feature Size',     # Dataset information
-        'models', 'epochs', 'batch size', "Polar Mode",             # Models information
-        'path', "cvnn version"                                      # Library information
+        'dataset', '# Classes', "Dataset Size", 'Feature Size',  # Dataset information
+        'models', 'epochs', 'batch size', "Polar Mode",  # Models information
+        'path', "cvnn version"  # Library information
     ]
     row_data = [
         iterations,
